//+------------------------------------------------------------------+
//|                                                          FVE.mq4 |
//|                               Copyright  2014, Gehtsoft USA LLC |
//|                                            http://fxcodebase.com |
//|                                      this version made by mladen |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, Gehtsoft USA LLC"
#property link      "http://fxcodebase.com"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Green
#property indicator_color2 Red

extern ENUM_TIMEFRAMES TimeFrame   = PERIOD_CURRENT;
extern int             Length1     = 22;
extern ENUM_MA_METHOD  MA1_Method  = MODE_SMA;
extern int             Length2     = 22;
extern ENUM_MA_METHOD  MA2_Method  = MODE_SMA;
extern bool            Interpolate = true;

double FVE[],Signal[],Intra[],Inter[],VE[],Vol[];
string indicatorFileName;
bool   returnBars;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   IndicatorDigits(Digits);
      SetIndexBuffer(0,FVE);
      SetIndexBuffer(1,Signal);
      SetIndexBuffer(2,Intra);
      SetIndexBuffer(3,Inter);
      SetIndexBuffer(4,VE);
      SetIndexBuffer(5,Vol);

      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);

   IndicatorShortName(timeFrameToString(TimeFrame)+" Finite Volume Elements ("+Length1+","+Length2+")");
   return(0);
}
int deinit() { return(0); }

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int pos,limit = MathMin(Bars-counted_bars,Bars-2);
         if (returnBars)  { FVE[0] = limit+1; return(0); } 

   //
   //
   //
   //
   //
   
   if (TimeFrame == Period())
   {  
      for(pos=limit; pos>=0; pos--)
      {
         Intra[pos] = MathLog(High[pos])-MathLog(Low[pos]);
         Inter[pos] = MathLog((High[pos]+Low[pos]+Close[pos])/3.0)-MathLog((High[pos+1]+Low[pos+1]+Close[pos+1])/3.0);
         Vol[pos]   = Volume[pos];
      } 
      for(pos=limit; pos>=0; pos--)
      {
         double Vintra = iStdDevOnArray(Intra, 0, Length1, 0, MODE_SMA, pos);
         double Vinter = iStdDevOnArray(Inter, 0, Length1, 0, MODE_SMA, pos);
         double Cutoff = 0.1*(Vintra+Vinter)*Close[pos];
         double MF     = Close[pos]-(High[pos]+Low[pos])/2.0+High[pos]+Low[pos]+Close[pos]-High[pos+1]-Low[pos+1]-Close[pos+1];
            VE[pos] = 0;
            if (MF >  Cutoff) VE[pos] =  Volume[pos];
            if (MF < -Cutoff) VE[pos] = -Volume[pos];
      }
      for(pos=limit; pos>=0; pos--)
      {
         double MA_VE = iMAOnArray(VE , 0, Length1, 0, MODE_SMA  , pos);
         double MA1   = iMAOnArray(Vol, 0, Length1, 0, MA1_Method, pos);
            FVE[pos]=0; 
            if (MA1!=0) FVE[pos]=100.*MA_VE/(MA1*Point);
      }  
      for(pos=limit; pos>=0; pos--) Signal[pos]=iMAOnArray(FVE, 0, Length2, 0, MA2_Method, pos);
      return(0);
   }
   
   //
   //
   //
   //
   //
   
    limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
    for(int i=limit; i >= 0; i--)  
    {
       int y = iBarShift(NULL,TimeFrame,Time[i]);
          FVE[i]    = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length1,MA1_Method,Length2,MA2_Method,0,y);
          Signal[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length1,MA1_Method,Length2,MA2_Method,1,y);
       if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

       //
       //
       //
       //
       //

       datetime time = iTime(NULL,TimeFrame,y);
          for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;
          for(int x = 1; x < n; x++) 
          {
             FVE[i+x]    = FVE[i]    + (FVE[i+n]    - FVE[i]   ) * x/n;
             Signal[i+x] = Signal[i] + (Signal[i+n] - Signal[i]) * x/n;
          }               
   }         
   return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}